      program test2
!
!       test differential equation
!       y''=-4y  with y(0)=1, y'(0)=0
!       solution:  y=cos(2x)
!
!       using 4th order Runge-Kutta,
!       adaptive Runge-Kutta 6th order
!       Numerov method
!
      implicit none

      external f2

      double precision x,h,hq
      double precision y(22),dy(22),yout(22)
      double precision xinitial, xfinal
      double precision pih
      double precision rkq
      double precision d12,fd12,den

      double precision xx(1250)
      double precision yy(1250)
      integer i, ix, n, ntot

      integer kread, kwrite
      data kread/5/, kwrite/6/

      data pih/1.570796326794897d0/
!
      n=2
      xinitial = 0.0d0
      xfinal   = 4.d0 * pih 
      h=0.05d0

      write (kwrite,*)
      write (kwrite,*) ' Test of Runge-Kutta'
      write (kwrite,*)  

!       initialize for rk4

      x=xinitial
      y(1)=1.d0      
      y(2)=0.d0

      ix=0

 100  continue

      call f2 (x,y,dy)

!      write (kwrite,10001)x,y(1),y(2)
10001 format(4d15.6)
!      call difsis (f2,y,x,n,h)

      call rk4 (y,dy,n,x,h,yout,f2)

      ix=ix+1
      xx(ix)=x+h
      yy(ix)=yout(1)

      x=x+h
      y(1)=yout(1)
      y(2)=yout(2)

      if (x.le.xfinal) go to 100 

      write (kwrite,*)
      write (kwrite,*) ' Result based on  Runge-Kutta (rk4):'

!      write (kwrite,*) ' final h:',h
      write (kwrite,*) '      x            y               error  '
 
 
      do i=1,ix
      write (kwrite,10001) xx(i),yy(i),(cos(2.d0*xx(i))-yy(i))/yy(i)
      end do

      write (kwrite,*)
      write (kwrite,*)
     

!---------------------------------------------------------------

!       initialize for difsis

      x=xinitial
      y(1)=1.d0
      y(2)=0.d0

      ix=0

 200  continue
      write (kwrite,10001) x,y(1),y(2),h

      call difsis (f2,y,x,n,h)

      ix=ix+1
      xx(ix)=x
      yy(ix)=y(1)

      if (x.le.xfinal) go to 200

      write (kwrite,*)
      write (kwrite,*) ' Result based on adaptive Runge-Kutta (difsis):'
      write (kwrite,*) ' final h:',h
      write (kwrite,*) '      x            y               error  '


      do i=1,ix
      write (kwrite,10001) xx(i),yy(i),(cos(2.d0*xx(i))-yy(i))/yy(i)
      end do

!-----------------------------------------------------------------

      write (kwrite,*)
      write (kwrite,*) ' Test of Numerov '

!        initialize  for Numerov

      h=0.01d0
      hq=h*h

      ntot=abs((xfinal-xinitial)/h)

      d12=1.d0/12.d0
      fd12=5.d0*d12

      xx(1)=xinitial
      xx(2)=xinitial+h
      yy(1)=cos(2.d0*xx(1))
      yy(2)=yy(1)

      rkq=4.d0

 
      do ix=2,ntot-1

        xx(ix+1)=xx(ix)+h

        den=1.d0+hq*d12*rkq

        yy(ix+1)= (2.d0*(1.d0-fd12*hq*rkq)*yy(ix) -   &
     &  (1.d0+d12*hq*rkq)*yy(ix-1))/den

       end do

      write (kwrite,*)
      write (kwrite,*) ' Result based on the Numerov procedure:'
      write (kwrite,*) ' ntot:',ntot
      write (kwrite,*) '      x            y               error  '

      do i=1,ntot
      write (kwrite,10001) xx(i),yy(i),(cos(2.d0*xx(i))-yy(i))/yy(i)
      end do


      stop 
      end 
